import axios from 'axios'
import type { AxiosRequestConfig } from 'axios'
export const apiPrefix = `/api/v1`
const token = localStorage.getItem("token") || ""
const service = axios.create({
    baseURL: '',
    timeout: 1800000,
    headers: { 'Content-Type': 'application/json', "Token":token }
})
export const setRequest = function(token:string) {
    if(token) {
        service.interceptors.request.use(config => {
            config.headers = config.headers || {}
            config.headers['Token'] = token
            let type: 'params' | 'data' = config.method == 'post' ? 'data' : 'params'
            return config
        }, (error) => {
            console.log("service.request:", error)
            return Promise.reject(error)
        })
    }
}

service.interceptors.response.use((response) => {
    return response.data
}, (error) => {
    // if(error.response && error.response.status == 401) {
    //     window.location.href = "/login"
    //     return
    // }
    // console.info("service.response:", error)
    return Promise.reject(error)
})

interface ResponseData<T> {
    code: number,
    message: string,
    data: T
}

const post = <T = any>(url: string, data?: any, config?: any): Promise<ResponseData<T>> => {
    return service.post(url, data, { ...config })
}

const get = <T = any>(url: string, data?: any, config?: AxiosRequestConfig): Promise<ResponseData<T>> => {
    return service.get(url, { params: data, ...config })
}

const put = <T = any>(url: string, data?: any, config?: AxiosRequestConfig): Promise<ResponseData<T>> => {
    return service.put(url, data, {...config} )
}

const del = <T = any>(url: string, data?: any, config?: AxiosRequestConfig): Promise<ResponseData<T>> => {
    return service.delete(url, {params:data, ...config })
}

export {post, get, put, del, service as request}

